@@ -1,6 +1,6 @@
 
-// Copyright Christopher Kormanyos 2002 - 2011.
-// Copyright 2011 John Maddock. Distributed under the Boost
+// Copyright Christopher Kormanyos 2002 - 2013.
+// Copyright 2011 - 2013 John Maddock. Distributed under the Boost
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -25,6 +25,14 @@ inline void pow_imp(T& result, const T& t, const U& p, const mpl::false_&)
 
    typedef typename boost::multiprecision::detail::canonical<U, T>::type int_type;
 
+   if(&result == &t)
+   {
+      T temp;
+      pow_imp(temp, t, p, mpl::false_());
+      result = temp;
+      return;
+   }
+
    // This will store the result.
    if(U(p % U(2)) != U(0))
    {
@@ -134,10 +142,6 @@ void hyp1F0(T& H1F0, const T& a, const T& x)
    // There are no checks on input range or parameter boundaries.
 
    typedef typename boost::multiprecision::detail::canonical<int, T>::type si_type;
-   typedef typename boost::multiprecision::detail::canonical<unsigned, T>::type ui_type;
-   typedef typename T::exponent_type exp_type;
-   typedef typename boost::multiprecision::detail::canonical<exp_type, T>::type canonical_exp_type;
-   typedef typename mpl::front<typename T::float_types>::type fp_type;
 
    BOOST_ASSERT(&H1F0 != &x);
    BOOST_ASSERT(&H1F0 != &a);
@@ -192,17 +196,16 @@ void eval_exp(T& result, const T& x)
    typedef typename boost::multiprecision::detail::canonical<int, T>::type si_type;
    typedef typename T::exponent_type exp_type;
    typedef typename boost::multiprecision::detail::canonical<exp_type, T>::type canonical_exp_type;
-   typedef typename boost::multiprecision::detail::canonical<float, T>::type float_type;
 
    // Handle special arguments.
    int type = eval_fpclassify(x);
    bool isneg = eval_get_sign(x) < 0;
-   if(type == FP_NAN)
+   if(type == (int)FP_NAN)
    {
-      result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
+      result = x;
       return;
    }
-   else if(type == FP_INFINITE)
+   else if(type == (int)FP_INFINITE)
    {
       result = x;
       if(isneg)
@@ -211,7 +214,7 @@ void eval_exp(T& result, const T& x)
          result = x;
       return;
    }
-   else if(type == FP_ZERO)
+   else if(type == (int)FP_ZERO)
    {
       result = ui_type(1);
       return;
@@ -224,17 +227,6 @@ void eval_exp(T& result, const T& x)
       xx.negate();
 
    // Check the range of the argument.
-   static const canonical_exp_type maximum_arg_for_exp = std::numeric_limits<number<T, et_on> >::max_exponent == 0 ? (std::numeric_limits<long>::max)() : std::numeric_limits<number<T, et_on> >::max_exponent;
-
-   if(xx.compare(maximum_arg_for_exp) >= 0)
-   {
-      // Overflow / underflow
-      if(isneg)
-         result = ui_type(0);
-      else
-         result = std::numeric_limits<number<T, et_on> >::has_infinity ? std::numeric_limits<number<T, et_on> >::infinity().backend() : (std::numeric_limits<number<T, et_on> >::max)().backend();
-      return;
-   }
    if(xx.compare(si_type(1)) <= 0)
    {
       //
@@ -318,7 +310,6 @@ void eval_log(T& result, const T& arg)
    // then let y = x - 1 and compute:
    // log(x) = log(2) * n + log1p(1 + y)
    //
-   typedef typename boost::multiprecision::detail::canonical<int, T>::type si_type;
    typedef typename boost::multiprecision::detail::canonical<unsigned, T>::type ui_type;
    typedef typename T::exponent_type exp_type;
    typedef typename boost::multiprecision::detail::canonical<exp_type, T>::type canonical_exp_type;
@@ -401,9 +392,6 @@ inline void eval_pow(T& result, const T& x, const T& a)
 {
    BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The pow function is only valid for floating point types.");
    typedef typename boost::multiprecision::detail::canonical<int, T>::type si_type;
-   typedef typename boost::multiprecision::detail::canonical<unsigned, T>::type ui_type;
-   typedef typename T::exponent_type exp_type;
-   typedef typename boost::multiprecision::detail::canonical<exp_type, T>::type canonical_exp_type;
    typedef typename mpl::front<typename T::float_types>::type fp_type;
 
    if((&result == &x) || (&result == &a))
@@ -411,6 +399,7 @@ inline void eval_pow(T& result, const T& x, const T& a)
       T t;
       eval_pow(t, x, a);
       result = t;
+      return;
    }
 
    if(a.compare(si_type(1)) == 0)
@@ -427,7 +416,18 @@ inline void eval_pow(T& result, const T& x, const T& a)
       result = x;
       return;
    case FP_ZERO:
-      result = si_type(1);
+      switch(eval_fpclassify(a))
+      {
+      case FP_ZERO:
+         result = si_type(1);
+         break;
+      case FP_NAN:
+         result = a;
+         break;
+      default:
+         result = x;
+         break;
+      }
       return;
    case FP_NAN:
       result = x;
@@ -435,13 +435,14 @@ inline void eval_pow(T& result, const T& x, const T& a)
    default: ;
    }
 
-   if(eval_get_sign(a) == 0)
+   int s = eval_get_sign(a);
+   if(s == 0)
    {
       result = si_type(1);
       return;
    }
 
-   if(a.compare(si_type(-1)) < 0)
+   if(s < 0)
    {
       T t, da;
       t = a;
@@ -451,7 +452,6 @@ inline void eval_pow(T& result, const T& x, const T& a)
       return;
    }
    
-   bool bo_a_isint = false;
    typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type an;
    T fa;
    try
@@ -469,9 +469,33 @@ inline void eval_pow(T& result, const T& x, const T& a)
       an = (std::numeric_limits<boost::intmax_t>::max)();
    }
 
-   if((eval_get_sign(x) < 0) && !bo_a_isint)
+   if((eval_get_sign(x) < 0))
    {
-      result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
+      typename boost::multiprecision::detail::canonical<boost::uintmax_t, T>::type aun;
+      try
+      {
+         eval_convert_to(&aun, a);
+         if(a.compare(aun) == 0)
+         {
+            fa = x;
+            fa.negate();
+            eval_pow(result, fa, a);
+            if(aun & 1u)
+               result.negate();
+            return;
+         }
+      }
+      catch(const std::exception&)
+      {
+         // conversion failed, just fall through, value is not an integer.
+      }
+      if(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+         result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
+      else
+      {
+         BOOST_THROW_EXCEPTION(std::domain_error("Result of pow is undefined or non-real and there is no NaN for this number type."));
+      }
+      return;
    }
 
    T t, da;
@@ -586,10 +610,7 @@ namespace detail{
    template <class T>
    void sinhcosh(const T& x, T* p_sinh, T* p_cosh)
    {
-      typedef typename boost::multiprecision::detail::canonical<int, T>::type si_type;
       typedef typename boost::multiprecision::detail::canonical<unsigned, T>::type ui_type;
-      typedef typename T::exponent_type exp_type;
-      typedef typename boost::multiprecision::detail::canonical<exp_type, T>::type canonical_exp_type;
       typedef typename mpl::front<typename T::float_types>::type fp_type;
 
       switch(eval_fpclassify(x))
