@@ -7,7 +7,7 @@
 #define BOOST_MATH_MP_TOMMATH_BACKEND_HPP
 
 #include <boost/multiprecision/number.hpp>
-#include <boost/multiprecision/rational_adapter.hpp>
+#include <boost/multiprecision/rational_adaptor.hpp>
 #include <boost/multiprecision/detail/integer_ops.hpp>
 #include <boost/math/special_functions/fpclassify.hpp>
 #include <boost/cstdint.hpp>
@@ -58,7 +58,7 @@ struct tommath_int
    }
    tommath_int& operator = (tommath_int&& o)
    {
-      mp_exch(&m_data, &o.data());
+      mp_exch(&m_data, &o.m_data);
       return *this;
    }
 #endif
@@ -93,11 +93,10 @@ struct tommath_int
    }
    tommath_int& operator = (long long i)
    {
-      BOOST_MP_USING_ABS
       if(m_data.dp == 0)
          detail::check_tommath_result(mp_init(&m_data));
       bool neg = i < 0;
-      *this = static_cast<unsigned long long>(abs(i));
+      *this = boost::multiprecision::detail::unsigned_abs(i);
       if(neg)
          detail::check_tommath_result(mp_neg(&m_data, &m_data));
       return *this;
@@ -315,10 +314,10 @@ struct tommath_int
       {
          int pos = result[0] == '-' ? 1 : 0;
          const char* pp = base == 8 ? "0" : "0x";
-         result.insert(pos, pp);
+         result.insert(static_cast<std::string::size_type>(pos), pp);
       }
       if((f & std::ios_base::showpos) && (result[0] != '-'))
-         result.insert(0, 1, '+');
+         result.insert(static_cast<std::string::size_type>(0), 1, '+');
       return result;
    }
    ~tommath_int()
@@ -609,6 +608,20 @@ inline unsigned eval_lsb(const tommath_int& val)
    return mp_cnt_lsb(const_cast< ::mp_int*>(&val.data()));
 }
 
+inline unsigned eval_msb(const tommath_int& val)
+{
+   int c = eval_get_sign(val);
+   if(c == 0)
+   {
+      BOOST_THROW_EXCEPTION(std::range_error("No bits were set in the operand."));
+   }
+   if(c < 0)
+   {
+      BOOST_THROW_EXCEPTION(std::range_error("Testing individual bits in negative values is not supported - results are undefined."));
+   }
+   return mp_count_bits(const_cast< ::mp_int*>(&val.data())) - 1;
+}
+
 template <class Integer>
 inline typename enable_if<is_unsigned<Integer>, Integer>::type eval_integer_modulus(const tommath_int& x, Integer val)
 {
@@ -639,7 +652,7 @@ template<>
 struct number_category<tommath_int> : public mpl::int_<number_kind_integer>{};
 
 typedef number<tommath_int >                     tom_int;
-typedef rational_adapter<tommath_int>               tommath_rational;
+typedef rational_adaptor<tommath_int>               tommath_rational;
 typedef number<tommath_rational>                 tom_rational;
 
 }}  // namespaces
