@@ -3,7 +3,7 @@
 
     http://www.boost.org/
 
-    Copyright (c) 2001-2010 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2012 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -57,11 +57,9 @@ struct bidirectional_map
 {
     typedef std::pair<FromType, ToType> value_type;
 
-// _MSC_FULL_VER == 160020506 || 160021003 detects the VC10 Beta 1 and 2 
-// compilers
 #if defined(BOOST_NO_POINTER_TO_MEMBER_TEMPLATE_PARAMETERS) || \
-    (defined(BOOST_MSVC) && ((BOOST_MSVC < 1300) || \
-        (_MSC_FULL_VER == 160020506 || _MSC_FULL_VER == 160021003))) || \
+    (defined(BOOST_MSVC) && \
+        ( (BOOST_MSVC < 1300) || (BOOST_MSVC == 1600) )) || \
     (defined(BOOST_INTEL_CXX_VERSION) && \
         (defined(_MSC_VER) && (BOOST_INTEL_CXX_VERSION <= 700))) 
 
@@ -184,7 +182,7 @@ private:
 public:
     bool has_pragma_once(std::string const &filename)
     {
-        using namespace boost::multi_index;
+        using boost::multi_index::get;
         return get<from>(pragma_once_files).find(filename) != pragma_once_files.end();
     }
     bool add_pragma_once_header(std::string const &filename, 
@@ -200,7 +198,7 @@ public:
         
         range_type r = pragma_once_files.get<to>().equal_range(guard_name);
         if (r.first != r.second) {
-            using namespace boost::multi_index;
+            using boost::multi_index::get;
             get<to>(pragma_once_files).erase(r.first, r.second);
             return true;
         }
@@ -275,7 +273,7 @@ bool include_paths::add_include_path (
 {
     namespace fs = boost::filesystem;
     if (path_) {
-    fs::path newpath = fs::complete(create_path(path_), current_dir);
+    fs::path newpath = util::complete_path(create_path(path_), current_dir);
 
         if (!fs::exists(newpath) || !fs::is_directory(newpath)) {
         // the given path does not form a name of a valid file system directory
@@ -336,7 +334,7 @@ bool include_paths::find_include_file (std::string &s, std::string &dir,
                 dirpath = create_path((*it).second);
                 dirpath /= create_path(s);
             }
-            
+
             dir = dirpath.string();
             s = normalize(currpath).string();    // found the required file
             return true;
@@ -401,20 +399,60 @@ include_paths::find_include_file (std::string &s, std::string &dir,
 ///////////////////////////////////////////////////////////////////////////////
 //  Set current directory from a given file name
 
+inline bool
+as_relative_to(boost::filesystem::path const& path, 
+    boost::filesystem::path const& base, boost::filesystem::path& result) 
+{
+    if (path.has_root_path()) {
+        if (path.root_path() == base.root_path()) 
+            return as_relative_to(path.relative_path(), base.relative_path(), result);
+
+        result = path;    // that's our result
+    } 
+    else {
+        if (base.has_root_path()) {
+            // cannot find relative path from a relative path and a rooted base
+            return false;
+        } 
+        else {
+            typedef boost::filesystem::path::const_iterator path_iterator;
+            path_iterator path_it = path.begin();
+            path_iterator base_it = base.begin();
+            while (path_it != path.end() && base_it != base.end() ) {
+                if (*path_it != *base_it) 
+                    break;
+                ++path_it; ++base_it;
+            }
+
+            for (/**/; base_it != base.end(); ++base_it)
+                result /= "..";
+
+            for (/**/; path_it != path.end(); ++path_it)
+                result /= *path_it;
+        }
+    }
+    return true;
+}
+
+///////////////////////////////////////////////////////////////////////////////
 inline
 void include_paths::set_current_directory(char const *path_)
 {
     namespace fs = boost::filesystem;
     
     fs::path filepath (create_path(path_));
-    fs::path filename = fs::complete(filepath, current_dir);
+    fs::path filename = util::complete_path(filepath, current_dir);
     if (fs::exists(filename) && fs::is_directory(filename)) {
+        current_rel_dir.clear();
+        if (!as_relative_to(filepath, current_dir, current_rel_dir))
+            current_rel_dir = filepath;
         current_dir = filename;
-        current_rel_dir = filepath;
     }
     else {
+        current_rel_dir.clear();
+        if (!as_relative_to(branch_path(filepath), current_dir, current_rel_dir))
+            current_rel_dir = branch_path(filepath);
         current_dir = branch_path(filename);
-        current_rel_dir = branch_path(filepath);
     }
 }
 
